<?php

namespace Drupal\graphql_fva\Plugin\Deriver\Fields;

use Drupal\Component\Plugin\PluginManagerInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Drupal\graphql\Utility\StringHelper;
use Drupal\graphql_views\Plugin\Deriver\ViewDeriverBase;
use Drupal\views\Plugin\views\display\DisplayPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Derive fields from configured views.
 */
class ViewDeriver extends ViewDeriverBase implements ContainerDeriverInterface {

  /**
   * The entity field manager instance.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * The entity type manager interface.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Interface plugin manager instance.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $interfacePluginManager;


  /**
   * {@inheritdoc}
   */
  public function __construct(EntityFieldManagerInterface $entity_field_manager, EntityTypeManagerInterface $entity_type_manager, PluginManagerInterface $interfacePluginManager) {
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->interfacePluginManager = $interfacePluginManager;
    parent::__construct($entity_type_manager, $interfacePluginManager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $basePluginId) {
    // Instantiates this form class.
    return new static(
    // Load the service required to construct this class.
      $container->get('entity_field.manager'),
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.graphql.interface')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($basePluginDefinition) {
    // Get entity type with views arguments entity field.
    $field_map = $this->entityFieldManager->getFieldMapByFieldType('views_arguments_entity_field_item');
    if (empty($field_map)) {
      return parent::getDerivativeDefinitions($basePluginDefinition);

    }

    if (!$this->entityTypeManager->hasDefinition('view')) {
      return parent::getDerivativeDefinitions($basePluginDefinition);
    }
    $viewStorage = $this->entityTypeManager->getStorage('view');
    foreach ($field_map as $entity_type => $field_map_entity_type) {
      foreach ($field_map_entity_type as $field_name => $field) {
        foreach ($field['bundles'] as $bundle) {
          if (empty($field_definition[$entity_type][$bundle])) {
            $field_definition[$entity_type][$bundle] = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
          }

          if (empty($field_definition[$entity_type][$bundle][$field_name])) {
            continue;
          }
          $settings = $field_definition[$entity_type][$bundle][$field_name]->getSettings();
          if (empty($settings['views_id'])) {
            continue;
          }
          list($view_id, $display_id) = explode('/', $settings['views_id']);
          /** @var \Drupal\views\ViewEntityInterface $view */
          $view = $viewStorage->load($view_id);
          if (empty($view) || !$this->getRowResolveType($view, $display_id)) {
            continue;
          }

          /** @var \Drupal\graphql_views\Plugin\views\display\GraphQL $display */
          $display = $this->getViewDisplay($view, $display_id);

          $id_graphql_views = implode('-', [$view_id, $display_id, 'view']);
          $id = implode('-', [
            'views_arguments_entity',
            $bundle,
            $field_name,
          ]);
          $info = $this->getArgumentsInfo($display->getOption('arguments') ?: []);
          $arguments = [];
          $arguments += $this->getContextualArguments($info, $id_graphql_views);
          $arguments += $this->getPagerArguments($display);
          $arguments += $this->getSortArguments($display, $id_graphql_views);
          $arguments += $this->getFilterArguments($display, $id_graphql_views);
          $suffix = call_user_func_array([
            StringHelper::class,
            'camelCase',
          ], [$entity_type, $bundle]);
          $name = call_user_func_array([
            StringHelper::class,
            'camelCase',
          ], ['viewsArgumentEntity', $field_name]);
          $this->derivatives[$id] = [
              'id' => $id,
              'name' => $name,
              'type' => $display->getGraphQLResultName(),
              'parents' => [$suffix],
              'arguments' => $arguments,
              'views_arguments' => $settings['views_arguments'],
              'view' => $view,
              'display' => $display_id,
              'paged' => $this->isPaged($display),
              'arguments_info' => $info,
            ] + $this->getCacheMetadataDefinition($view, $display) + $basePluginDefinition;
        }
      }
    }


    return parent::getDerivativeDefinitions($basePluginDefinition);
  }

  /**
   * Helper function to return the contextual filter argument if any exist.
   *
   * @param array $arguments
   *   The array of available arguments.
   * @param string $id
   *   The plugin derivative id.
   *
   * @return array
   *   The contextual filter argument if applicable.
   */
  protected function getContextualArguments(array $arguments, $id) {
    if (!empty($arguments)) {
      return [
        'contextualFilter' => [
          'type' => StringHelper::camelCase($id, 'contextual', 'filter', 'input'),
        ],
      ];
    }

    return [];
  }

  /**
   * Helper function to retrieve the sort arguments if any are exposed.
   *
   * @param \Drupal\views\Plugin\views\display\DisplayPluginInterface $display
   *   The display plugin.
   * @param string $id
   *   The plugin derivative id.
   *
   * @return array
   *   The sort arguments if any exposed sorts are available.
   */
  protected function getSortArguments(DisplayPluginInterface $display, $id) {
    $sorts = array_filter($display->getOption('sorts') ?: [], function ($sort) {
      return $sort['exposed'];
    });
    return $sorts ? [
      'sortDirection' => [
        'type' => 'ViewSortDirection',
        'default' => 'asc',
      ],
      'sortBy' => [
        'type' => StringHelper::camelCase($id, 'sort', 'by'),
      ],
    ] : [];
  }

  /**
   * Helper function to return the filter argument if applicable.
   *
   * @param \Drupal\views\Plugin\views\display\DisplayPluginInterface $display
   *   The display plugin.
   * @param $id
   *   The plugin derivative id.
   *
   * @return array
   *   The filter argument if any exposed filters are available.
   */
  protected function getFilterArguments(DisplayPluginInterface $display, $id) {
    $filters = array_filter($display->getOption('filters') ?: [], function ($filter) {
      return array_key_exists('exposed', $filter) && $filter['exposed'];
    });

    return !empty($filters) ? [
      'filter' => [
        'type' => $display->getGraphQLFilterInputName(),
      ],
    ] : [];
  }

  /**
   * Helper function to retrieve the pager arguments if the display is paged.
   *
   * @param \Drupal\views\Plugin\views\display\DisplayPluginInterface $display
   *   The display plugin.
   *
   * @return array
   *   An array of pager arguments if the view display is paged.
   */
  protected function getPagerArguments(DisplayPluginInterface $display) {
    return $this->isPaged($display) ? [
      'page' => ['type' => 'Int', 'default' => $this->getPagerOffset($display)],
      'pageSize' => [
        'type' => 'Int',
        'default' => $this->getPagerLimit($display),
      ],
    ] : [];
  }

  /**
   * Helper function to retrieve the types that the view can be attached to.
   *
   * @param array $arguments
   *   An array containing information about the available arguments.
   *
   * @return array
   *   An array of additional types the view can be embedded in.
   */
  protected function getTypes(array $arguments) {
    $types = ['Root'];

    if (empty($arguments)) {
      return $types;
    }

    foreach ($arguments as $argument) {
      // Depending on whether bundles are known, we expose the view field
      // either on the interface (e.g. Node) or on the type (e.g. NodePage)
      // level. Here we specify types managed by other graphql_* modules,
      // yet we don't define these modules as dependencies. If types are not
      // in the schema, the resulting GraphQL field will be attached to
      // nowhere, so it won't go into the schema.
      if (empty($argument['bundles']) && empty($argument['entity_type'])) {
        continue;
      }

      if (empty($argument['bundles'])) {
        $types = array_merge($types, [StringHelper::camelCase($argument['entity_type'])]);
      }
      else {
        $types = array_merge($types, array_map(function ($bundle) use ($argument) {
          return StringHelper::camelCase($argument['entity_type'], $bundle);
        }, array_keys($argument['bundles'])));
      }
    }

    return $types;
  }

}
